<?php
/**
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @package REPHPGraph
 */

/**
 * Legfelső absztrakt ősosztály
 * 
 * Minden további osztály közvetve ennek az osztálynak a leszármazottja.<br /> 
 *  
 * @version 1.0
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @package REPHPGraph
 */
abstract class AGraph
{
	/**
	 * Képazonosító	
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @var int $source 
	 */
	protected $source;
	/**
	 * Kép háttérszíne
	 * 
	 * @var int
	 */
	protected $backGround;
	/**
	 * Kép magassága
	 * 
	 * @var int
	 */
	protected $height   = 0;        
	/**
	 * Kép szélessége
	 * 
	 * @var int
	 */
	protected $width    = 0;
	/**
	 * Kép típusa
	 * 
	 * Engedélyezett típusok: {@link $types}
	 * 
	 * @var string
	 */
	protected $type     = 'jpeg';
	/**
	 * Támogatott képtípusok
	 *  
	 * csak olvasható (readonly)
	 * 
	 * @var array
	 */
	protected $types    = array(1=>'gif','jpeg','jpg','png');
	/**
	 * Kép minősége
	 * 
	 * Ennek tulajdonságnak csak JPEG esetén van értelme.<br />
	 * GIF vagy PNG minőségét nem lehet állítani<br />
	 * 0 <= $quality <= 100
	 * 
	 * @var int
	 */
    	protected $quality = 100;                
	/**
	 * Kép átlátszósága
	 * 
	 * Értéke: false | true
	 * 
	 * @var boolean
	 */
	protected $transparent = false;
        
	/**
	 * Beállítja a tulajdonságok értékét
	 * 
	 * A __set() metódus akkor fut le, amikor egy változónak értéket prbálunk adni.<br />
	 * Ezzel korlátozható a tulajdonságok írási joga
	 * 
	 * @param string $var Változó neve
	 * @param mixed $value Változó értéke
	 * @return void
	 */
	function __set($var,$value)
	{
		switch ($var) {
			case 'quality': 
				$value = (int)$value;
				if ($value < 0) $value = 0;
				else if($value > 100) $value=100;
				$this->quality = $value;		
				break;
			case 'height': case 'width': case 'backGround':
				$value = (int)$value;
				if ($value < 0) $value = 0;
				$this->$var = $value;
				break;
			case 'type':
				$value = strtolower($value);
				if (in_array($value,$this->types)) {
					$this->type = $value;
				} else {
					throw new SetTypeException("Hibás típus");
				}
				break;
			case 'transparent':
				$this->$var = ($value === true or $value == 'true') ? true : false;
			default: $this->$var = $value;
		}
	}
	/**
	 * Változok értékeinek lekérdezése
	 * 
	 * A __get() metódus akkor fut le, amikor egy változó értékét próbáljuk lekérdezni.<br />
	 * Ezzel korlátozható a tulajdonságok olvasási joga
	 * 
	 * @param string $var
	 * @return mixed
	 */
 	function __get($var)
	{
		return $this->$var;
	}
	
	/**
	 * Színek előállítása
	 * 
	 * Ez a metóus három féle képpen használható. Egyik módja, ha megadjuk a kívánt szín RGB értékeit,
	 * három külön paraméterben így:
	 * <code>
	 * $color = $graph->createColor(0,0,255);
	 * </code>
	 * Ami egy kék színt hoz létre. A másik megoldás, hogy megadjuk az első paraméterben stringként,
	 * vesszőkkel elválasztva az egyes színkomponenseket:
	 * <code>
	 * $color = $graph->createColor("0,0,255");
	 * </code> 
	 * És végül az utolsó módszer, a hexadecimális értékmegadás:
	 * <code>
	 * $color = $graph->createColor("#0000FF");
	 * </code>
	 * 
	 * @param mixed $color int | string 
	 * @param int $greenc
	 * @param int $bluec
	 * @return int
	 */
	public function createColor($color,$greenc=null,$bluec=null)
	{
		//ha mind a három paraméter meg van adva
		if($bluec !== null and $greenc !== null) {
			$red = $color;
			$green = $greenc;
			$blue = $bluec;
		//ha csak az első paraméter van megadva
		} else {
			//akkor ha # jellel kezdődik
			if(substr($color,0,1) == '#') {
				//hexadecimális formátumnak tekinti.
				//felbontja 3 részre és decimálisba váltja a részeket
				$red_hex = substr($color,1,2);
				$green_hex = substr($color,3,2);
				$blue_hex = substr($color,5,2);
				$red = hexdec($red_hex);
				$green = hexdec($green_hex);
				$blue = hexdec($blue_hex);
			//egyébként
			} else {
				//vesszük mentén 3 részre vágja a szánt (rgb)
				$color = explode(',',$color);
				$red = $color[0];
				$green = $color[1];
				$blue = $color[2];
			}
		}
		//szín generálása
		return imageColorAllocate($this->source,$red,$green,$blue);
	}
}
?>