<?php
/**
 * R.E. PHP Graph v1.0
 * 
 * Ez a program megkönnyíti a bonyolultabb alakzatok és megoldások generálását PHP-ben képekre.<br /> 
 * Alapvetően 3 féle típus támogatott. GIF, JPEG és PNG<br />
 * A generált képek megjeleníthetők a böngészőben, vagy lementhetők a webszerverre, illetve a kliens gépre is.
 * 
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @version 1.0
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * 
 * @example examples/fullExample.php Teljes példa 
 * @package REPHPGraph
 */
        
/**
 * Szükséges függvények
 */
require_once dirname(__FILE__)."/functions.php";
 
/**
 * Agraph importálása
 */
require_once dirname(__FILE__)."/AGraph.class.php";

/**
 * FileExistsException ipmortálása
 */
require_once dirname(__FILE__)."/exceptions/FileExistsException.class.php";

/**
 * DirExistsException ipmortálása
 */
require_once dirname(__FILE__)."/exceptions/DirExistsException.class.php";

/**
 * Grafikát vezérlő osztály
 * 
 * Ebből az osztályból példányosított objektum segítségével hozható létre fő kép,<br />
 * amire ráhelyezhetők a további alakzatok, minták, képek.<br />
 * Lehetőséget nyújt az eredmény szerverre, valamint kliens gépre mentésére, illetve a böngészőben megjelenítésre.<br />
 * Tartalmaz továbbá egy, a színek létrehozására szolgáló {@link createColor()} metódust is.
 * <br />
 * Üres fekete kép létrehozása ( 200 x 200 )
 * <code>
 * <?php
 * $graph = new Graph(200,200);
 * $graph->setBackground(0,0,0);
 * $graph->flush();
 * ?>
 * </code>
 * 
 * @version 1.1
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @package REPHPGraph
 */
class Graph extends AGraph 
{ 
	/**
	 * Képre felvitt alakzatok erőforrás azonosítóinak listája
	 * 
	 * @var array
	 */
	protected $objects = array();

	/**
	 * Grafikus elemek importja
	 *
	 * @param mixed $objects Importálandó grafikus elem osztályok neve 
	 * 						(Ha egy osztály, lehet string. Több esetén tömb
	 * 						illetve paraméter nélkül hívva az összes grafikus elem)
	 */
	public static function importObject($objects=null) {
		self::import('objects',$objects);
	}

	/**
	 * Iterátorok importja
	 *
	 * @param mixed $iterators Importálandó iterátor osztályok neve 
	 * 						(Ha egy osztály, lehet string. Több esetén tömb
	 * 						illetve paraméter nélkül hívva az összes grafikus elem)
	 */
	public static function importIterator($iterators=null) {
		self::import('iterators',$iterators);
	}
	/**
	 * Osztályok importja
	 *
	 * @param string $dir Mappa neve, ahol az importálandó osztályok vannak
	 * @param mixed $objects Importálandó osztályok neve
	 * 						(Ha egy osztály, lehet string. Több esetén tömb
	 * 						illetve paraméter nélkül hívva az összes grafikus elem)
	 */
	public static function import($dir,$classes=null) {
		$dir = dirname(__FILE__).'/'.trim($dir,'/');
		if (!is_dir($dir)) {
			throw new DirExistsException("Nincs ilyen könyvtár: ".$dir);
		}
		$all = false;
		if (is_string($classes)) {
			$classes = array($classes); 
		} else if (!$classes) {
			$classes = scandir($dir);
			$all=true;
		}
		foreach ($classes as $object) {
			if ($object != '.' and $object != '..' and !is_dir($dir.'/'.$object)) {
				if (!$all) { $object .= '.class.php'; }
				else if ( end(explode('.',$object)) != 'php' ) {
					continue;
				}
				if (!file_exists($dir.'/'.$object)) {
					throw new FileExistsException("Nincs ilyen grafikus elem: $dir/".$object);
				}
				require_once $dir.'/'.$object;
			}
		}
	}
	
	/**
	 * Kép háttérszínének lekérdezése
	 * @return int
	 */
	public function getBackground()
	{
		return $this->background;
	}
	/**
	 * Kép háttérszínének beállítása
	 * 
	 * Paraméterezése és működése megegyezik a {@link createColor()} 
	 * metódus paraméterezésével működésével.
	 * 
	 * @param int $color
	 * @param int $greenc
	 * @param int $bluec
	 */
	public function setBackground($color,$greenc=null,$bluec=null) 
	{
		$this->backGround = $this->createColor($color,$greenc,$bluec);	
	}
        	
	/**
	 * Létrehoz egy üres képet $width szélességgel és $height magassággal
	 * 
	 * @param int $width Kép szélessége
	 * @param int $height Kép magassága
	 */
	public function __construct($width,$height)
	{
		$this->width = $width;
		$this->height = $height;
		$this->source = imagecreatetruecolor($width,$height);
	}
	
	/**
	 * Objektumok hozzáadása a képhez
	 * 
	 * Minden újabb képet, vagy alakzatot át kell adni a Graph osztályból példányosított objektumnak.<br />
	 * Ez a metódus meghívja az objektum {@link IObject::create()} metódusát, majd létrehozza
	 * a megfelelő típusú képet. Ennek a képnek az erőforrásazonosítóját hozzáadja a az {@link $objects} listához.
	 * 
	 * @param AObject $object
	 */
	function add($object)
	{
		ob_start();
		//háttér kirajzolása
		imagefilledrectangle($object->source,0,0,$object->width-1, $object->height-1, $object->backGround);
		//ha a transparent tulajdonság true, akkor háttérszín beállítása áttetszővé
		if ($object->transparent === true) {
			imageColorTransparent($object->source,$object->backGround);
		}
		//alakzat irajzolása
		$object->create();
		//típus meghatározás
		if (!array_search($object->type,$object->types,true) or $object->type == "jpg") {
			$object->type = 'jpeg';
		}
                
		//kép létrehozása
		if ($object->transparent === true) {
			imagegif($object->source); 
		} else {
			$this->createImage($object,'image'.$object->type); 
		}
		//kimenet elfogásával az erőforrásazonosító betöltése az $objects listába
		$image = ob_get_clean();
		$object->source = imageCreateFromString($image); 
		$this->objects[] = $object;	
	}
	
	/**
	 * Felépíti az $objects listából a képet
	 * 
	 * @return string imageGif | imageJpeg | imagePng
	 */    
	private function buildImage() 
	{
		//háttérszínnel kitölti a fő képet
		imagefilledrectangle($this->source, 0,0, $this->width-1, $this->height-1, $this->backGround);
		//szükség esetén áttetszővé teszi a hátteret
		if ($this->transparent === true ) {
			imageColorTransparent($this->source, $this->backGround);
		}
		//az $objects tömb összes elemét rámásolja a fő képre
		foreach($this->objects as $key => $object) {
			imagecopy($this->source,$object->source,$object->x - ($object->width / 2),$object->y - ($object->height / 2),0,0,$object->width,$object->height);	
		}
		//típus beállítása
		if (!array_search($this->type,$this->types,true) or $this->type == "jpg") {
			$this->type = 'jpeg';
		}
		return 'image'.$this->type;       
	}        
        
	/**
	 * Kép létrehozása
	 * 
	 * Létrehoz egy képet az $object objektumból a $func függvény segítségével.<br />
	 * Szükség esetén az $src néven lementi a képet a szerveren
	 * 
	 * @param Object $object AObject | Graph
	 * @param string $func képet létrehozo függvény neve
	 * @param string $src kép helye mentés esetén
	 */
	private function createImage($object,$func,$src=null) 
	{
		//ha nem gif és nem png képről van szó, akkor a minőséget is be lehet állítani
		if(strtolower($object->type) != 'gif' and strtolower($object->type) != 'png')
		{
			$func($object->source,$src,$object->quality);
		}else{
			$func($object->source,$src);
		}                
	}  
	/**
	 * Kép küldése a kimenetre. 
	 */
	function flush()
	{
		//kép összeállítása
		$func = $this->buildImage();
		//kimenet típusának beállítása
		header("Content-type: image/".$this->type);
		//kimenetre küldés	
		$this->createImage($this,$func);
	}

	/**
	 * Kép felkínálása letöltésre a kliensen
	 * 
	 * @param string $src Letöltés után milyen néven legyen mentve a kép
	 */
	function saveClient($src)
	{
		//kép összeállítása
		$func = $this->buildImage();
		//kimenet tipusának beállítása, és letöltésre állítás
		header("Content-type: image/".$this->type);
		header('Content-Disposition: attachment; filename="'.trim(htmlentities(basename($src))).'"'); 
		//küldés a kimenetre
		$this->createImage($this,$func);
	}	
	/**
	 * Kép mentése a szerveren
	 * 
	 * @param string $src Mentés helye
	 */
	function save($src)
	{
		$this->createImage($this,$this->buildImage(),$src);
	}		
}

?>
