<?php
/**
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @package REPHPGraph
 */

/**
 * Aobject importálása
 */
require_once dirname(__FILE__)."/../AObject.class.php";

/**
 * IObject importálása
 */
require_once dirname(__FILE__)."/../interfaces/IObject.class.php";

/**
 * FileExistsException ipmortálása
 */
require_once dirname(__FILE__)."/../exceptions/FileExistsException.class.php";

/**
 * ImageTypeException importálása
 */
require_once dirname(__FILE__)."/../exceptions/ImageTypeException.class.php";

/**
 * Kép osztály
 * 
 * Már előre elkészített képeket lehet ráhelyezni objektumként a készítendő képre.<br />
 * A képet szintén elforgathatjuk, mint az összes többi objektumot, ám előfordulhat, hogy
 * az adott PHP beállítás nem támogatja a forgatást. Ebben az esetben használja a program a korlátozott
 * szögelfordulást támogató {@link imageRotate()} függvényt. 
 * 
 * {@example ../examples/image.php}
 * 
 * @version 1.0
 * @package REPHPGraph
 */
class Image extends AObject implements IObject
{
	/**
	 * Segédváltozó a típusokhoz
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @ignore
	 * @var array
	 */
	protected $flags = array(1=>'GIF','JPEG','PNG','SWF'); 
	/**
	 * Fájl kiterjesztése
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @var string
	 */
	protected $ext;	
        
	/**
	 * Kép url-je
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @var string
	 */
	protected $src = '';
	
	/**
	 * honnan vágjon a képből Y
	 * 
	 * @ignore
	 * @var int
	 */
	protected $cutY1 = 0;
	/**
	 * honnan vágjon a képből X
	 * 
	 * @ignore
	 * @var int
	 */
	protected $cutX1 = 0;
	/**
	 * meddig vágjon a képből Y
	 * 
	 * @ignore
	 * @var int
	 */
	protected $cutY2 = 0;
	/**
	 * meddig vágjon a képből X
	 * 
	 * @ignore
	 * @var int
	 */
	protected $cutX2 = 0;
	/**
	 * jobbra balra vagy középre igazítson
	 * 
	 * @ignore
	 * @var int
	 */
	protected $Jx = 0;
	/**
	 * fel le vagy középre igazitson
	 * 
	 * @ignore
	 * @var int
	 */
	protected $Jy = 0;	
	/**         
	 *  
	 * Képosztály konstruktora
	 * 
	 * Hibás $src paraméter esetén {@link ImageTypeException} -t vált ki
	 * 
	 * @param string $src Kp url-je
	 */	
	function Image($src)
	{
		$this->src = $src;
		if(file_exists($src)) {
			$info = getimagesize($src);
			$this->width = $info[0];
			$this->height = $info[1];
			if(array_key_exists(strtoupper($info[2]),$this->flags)) {
				$this->ext = $this->flags[$info[2]];	
			} else {
				$this->ext = 'jpeg';
			}
			if (!in_array(strtolower($this->type),$this->types)) {
				$types = implode(', ',$this->types);
				throw new ImageTypeException("Nem támogatott képtípus! Támogatott: ".$types);
			}
		} else {
			throw new FileExistsException("Nem található <b>".$src."</b> nevű képfájl!");
		}
                
		$this->source = imageCreateTrueColor($this->width,$this->height);
				
	}
	/**
	 * Szülőosztályban még nem definiált változó lekérdezések
	 * 
	 * @param string $var
	 * @return mixed
	 */
	function __get($var) 
	{
		switch ($var)
		{
			case 'cutxX1': case 'cutX2': case 'cutY1': case 'cutY2': 
			case 'Jx': case 'Jy': case 'flags':
				break;
			default: return parent::__get($var);
		}
	}
        
	/**
	 * Kép X koordinátájának beállítása
	 * 
	 * $Jx megadásával beállítható, hogy a kép X koordinátáját honnan számolja
	 * <b>$Jx értékei</b>
	 * <ul>
	 * <li>0: A kép bal oldalától</li>
	 * <li>1: A kép közepétől</li>
	 * <li>2: A kép jobb oldalától</li>
	 * </ul> 
	 * 
	 * @param int $x X koordináta
	 * @param int $Jx 0 | 1 | 2
	 */
	function setX($x,$Jx = 0)
	{
		if($Jx == 0) {
			$eltol = ceil($this->width / 2);
		} else if ($Jx == 1) {
			$eltol = 0;
		} else {
			$eltol = 0-ceil($this->width / 2);
		}
		$this->x = $eltol + $x;
		$this->Jx = $Jx;
	}
	/**
	 * Kép Y koordinátájának beállítása
	 * 
	 * $Jy megadásával beállítható, hogy a kép Y koordinátáját honnan számolja
	 * <b>$Jy értékei</b>
	 * <ul>
	 * <li>0: A kép bal tetejétől</li>
	 * <li>1: A kép közepétől</li>
	 * <li>2: A kép aljától</li>
	 * </ul>
	 * 
	 * @param int $yY koordináta
	 * @param int $Jy 0 | 1 | 2
	 */
	function setY($y,$Jy = 0)
	{
		if($Jy == 0) {
			$eltol = ceil($this->height / 2);
		} else if ($Jy == 1) {
			$eltol = 0;
		} else {
			$eltol = 0-ceil($this->height / 2);
		}
		$this->y = $eltol + $y;
		$this->Jy = $Jy;
	}
	/**
	 * Kép létrehozása
	 *
	 * Létrehozza a képet, amit aztán majd a {@link Graph::add()} metódus rámásol a készítendő képre
	 * 
	 */
	function create()
	{
		if (file_exists($this->src)) {
			$func = 'imageCreateFrom'.$this->ext;
			$new_src = $func($this->src);
			
			if($this->cutX1 > 0 or $this->cutY1 > 0 or $this->cutX2 > 0 or $this->cutY2 > 0) {
				if($this->cutX1 <= 0) {
					$this->cutX1 = 0;
				}
				if($this->cutY1 <= 0) {
					$this->cutY1 = 0;
				}
				if($this->cutX2 <= 0) {
					$this->cutX2 = $this->width;	
				}
				if($this->cutY2 <= 0) {
					$this->cutY2 = $this->height;
				}
				$width = $this->width;
				$height = $this->height;
				$this->width = abs((int)$this->cutX2 - (int)$this->cutX1);
				$this->height = abs((int)$this->cutY2 - (int)$this->cutY1);
				
				$this->resetXY($width,$height);
				
				$this->source = ImageCreateTrueColor($this->width,$this->height);
				imageCopyResized($this->source,$new_src,0,0,$this->cutX1,$this->cutY1,$this->width,$this->height,$this->width,$this->height);
			} else {
			
				$this->source = $new_src;
			}
			if($this->angle != 0) {
				$new_src = imagerotate($this->source,$this->angle,$this->backGround);
				$this->width = imagesx($new_src);
				$this->height = imagesy($new_src);
				$this->source = ImageCreateTrueColor($this->width,$this->height);
				imageCopyResized($this->source,$new_src,0,0,0,0,$this->width,$this->height,$this->width,$this->height);
			}
		}
	}
	/**
	 * Kép részletének kivágása
	 * 
	 * Ezzel a metódussal a forrás kép egy részletét lehet kivágni, 
	 * így nem az egész kép kerül majd másolásra
	 * 
	 * @param int $x1 Bal felső pont X koordinátája
	 * @param int $y1 Bal felső pont Y koordinátája
	 * @param int $x2 Jobb alsó pont X koordinátája
	 * @param int $y2 Jobb alsó pont Y koordinátája
	 */
	function cut($x1,$y1,$x2,$y2) 
	{
		$this->cutX1 = $x1;
		$this->cutY1 = $y1; 
		
		$this->cutY2 = $y2;
		$this->cutX2 = $x2;
	}
	
	/**
	 * Poziciók újraszámolása
	 * 
	 * @param int $width Kép új szélessége
	 * @param height $height Kép új magassága
	 * @ignore 
	 */
	protected function resetXY($width,$height)
	{ 	
		switch($this->Jx) {
			case 0: $this->setX($this->x - abs($width /2 ),$this->Jx);
				break;
			case 1: break;
			default: $this->setX($this->x + abs($width /2 ),$this->Jx);
				break;
		}
		switch($this->Jy) {
			case 0: $this->setY($this->y - abs($height /2 ),$this->Jy);
				break;
			case 1: break;
			default: $this->setY($this->y + abs($height /2 ),$this->Jy);
				break;
		}
	}
}
?>